/* Definitions of target machine for GNU compiler, for IBM S/390
   Copyright (C) 2002 Free Software Foundation, Inc.
   Contributed by Hartmut Penner (hpenner@de.ibm.com) and
                  Ulrich Weigand (uweigand@de.ibm.com).
This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* Add any extra modes needed to represent the condition code.  */

CC (CCZ)
CC (CCA)
CC (CCAP)
CC (CCAN)
CC (CCL)
CC (CCL1)
CC (CCL2)
CC (CCU)
CC (CCUR)
CC (CCS)
CC (CCSR)
CC (CCT)
CC (CCT1)
CC (CCT2)
CC (CCT3)
